@@ -39,6 +39,7 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-pango-extensions.h>
 #include <eel/eel-string.h>
+#include <eel/eel-accessibility.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtksignal.h>
 #include <libart_lgpl/art_rgb.h>
@@ -47,6 +48,9 @@
 #include <libart_lgpl/art_svp_vpath.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomecanvas/gnome-canvas-util.h>
+#include <atk/atkimage.h>
+#include <atk/atkcomponent.h>
+#include <atk/atknoopobject.h>
 #include <math.h>
 #include <stdio.h>
 #include <string.h>
@@ -60,10 +64,6 @@
 #define MAX_TEXT_WIDTH_STANDARD 135
 #define MAX_TEXT_WIDTH_TIGHTER 80
 
-/* The list of characters that cause line breaks can be localized. */
-static const char untranslated_line_break_characters[] = N_(" -_,;.?/&");
-#define LINE_BREAK_CHARACTERS _(untranslated_line_break_characters)
-
 /* Private part of the NautilusIconCanvasItem structure. */
 struct NautilusIconCanvasItemDetails {
 	/* The image, text, font. */
@@ -102,17 +102,19 @@ struct NautilusIconCanvasItemDetails {
 	ArtIRect canvas_rect;
 	ArtIRect text_rect;
 	ArtIRect emblem_rect;
+
+	/* Accessibility bits */
+	GailTextUtil *text_util;
 };
 
 /* Object argument IDs. */
 enum {
-	ARG_0,
-	ARG_EDITABLE_TEXT,
-	ARG_ADDITIONAL_TEXT,
-    	ARG_HIGHLIGHTED_FOR_SELECTION,
-    	ARG_HIGHLIGHTED_AS_KEYBOARD_FOCUS,
-    	ARG_HIGHLIGHTED_FOR_DROP,
-    	ARG_MODIFIER,
+	PROP_0,
+	PROP_EDITABLE_TEXT,
+	PROP_ADDITIONAL_TEXT,
+    	PROP_HIGHLIGHTED_FOR_SELECTION,
+    	PROP_HIGHLIGHTED_AS_KEYBOARD_FOCUS,
+    	PROP_HIGHLIGHTED_FOR_DROP,
 };
 
 typedef enum {
@@ -131,17 +133,7 @@ typedef struct {
 	GList *emblem;
 } EmblemLayout;
 
-enum {
-	BOUNDS_CHANGED,
-	LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL];
-
 static  gboolean antialias_selection_rectangle = TRUE;
-static	guint32 highlight_background_color = EEL_RGBA_COLOR_PACK (0x00, 0x00, 0x00, 0xFF);
-static	guint32 highlight_text_color	   = EEL_RGBA_COLOR_PACK (0xFF, 0xFF, 0xFF, 0xFF);
-static  guint32 highlight_text_info_color  = EEL_RGBA_COLOR_PACK (0xCC, 0xCC, 0xCC, 0xFF);
 
 static int click_policy_auto_value;
 
@@ -189,20 +181,26 @@ static void     clear_rounded_corners                (GdkPixbuf
 						      GdkPixbuf                     *corner_pixbuf,
 						      int                            corner_size);
 
-EEL_CLASS_BOILERPLATE (NautilusIconCanvasItem,
-		       nautilus_icon_canvas_item,
-		       GNOME_TYPE_CANVAS_ITEM)
+
+static NautilusIconCanvasItemClass *parent_class = NULL;
 
 /* Object initialization function for the icon item. */
 static void
 nautilus_icon_canvas_item_init (NautilusIconCanvasItem *icon_item)
 {
+	static gboolean setup_auto_enums = FALSE;
 	NautilusIconCanvasItemDetails *details;
 
+	if (!setup_auto_enums) {
+		eel_preferences_add_auto_enum
+			(NAUTILUS_PREFERENCES_CLICK_POLICY,
+			 &click_policy_auto_value);
+		setup_auto_enums = TRUE;
+	}
+
 	details = g_new0 (NautilusIconCanvasItemDetails, 1);
 
 	icon_item->details = details;
-
 	nautilus_icon_canvas_item_invalidate_label_size (icon_item);
 }
 
@@ -218,6 +216,11 @@ nautilus_icon_canvas_item_finalize (GObject *object)
 	if (details->pixbuf != NULL) {
 		g_object_unref (details->pixbuf);
 	}
+	
+	if (details->text_util != NULL) {
+		g_object_unref (details->text_util);
+	}
+
 	eel_gdk_pixbuf_list_free (details->emblem_pixbufs);
 	g_free (details->editable_text);
 	g_free (details->additional_text);
@@ -226,6 +229,14 @@ nautilus_icon_canvas_item_finalize (GObject *object)
 	if (details->rendered_pixbuf != NULL) {
 		g_object_unref (details->rendered_pixbuf);
 	}
+
+	if (details->editable_text_layout != NULL) {
+		g_object_unref (details->editable_text_layout);
+	}
+
+	if (details->additional_text_layout != NULL) {
+		g_object_unref (details->additional_text_layout);
+	}
 	
 	g_free (details);
 
@@ -263,61 +274,75 @@ nautilus_icon_canvas_item_invalidate_label_size (NautilusIconCanvasItem *item)
 	}
 }
 
-/* Set_arg handler for the icon item. */
+/* Set property handler for the icon item. */
 static void
-nautilus_icon_canvas_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+nautilus_icon_canvas_item_set_property (GObject        *object,
+					guint           property_id,
+					const GValue   *value,
+					GParamSpec     *pspec)
 {
 	NautilusIconCanvasItem *item;
 	NautilusIconCanvasItemDetails *details;
 
-	item =  NAUTILUS_ICON_CANVAS_ITEM (object);
+	item = NAUTILUS_ICON_CANVAS_ITEM (object);
 	details = item->details;
 
-	switch (arg_id) {
+	switch (property_id) {
 
-	case ARG_EDITABLE_TEXT:
-		if (eel_strcmp (details->editable_text, GTK_VALUE_STRING (*arg)) == 0) {
+	case PROP_EDITABLE_TEXT:
+		if (eel_strcmp (details->editable_text,
+				g_value_get_string (value)) == 0) {
 			return;
 		}
 
 		g_free (details->editable_text);
-		details->editable_text = g_strdup (GTK_VALUE_STRING (*arg));
+		details->editable_text = g_strdup (g_value_get_string (value));
+		if (details->text_util) {
+			gail_text_util_text_setup (details->text_util,
+						   details->editable_text);
+		}
 		
-		nautilus_icon_canvas_item_invalidate_label_size (item);		
+		nautilus_icon_canvas_item_invalidate_label_size (item);
 		break;
 
-	case ARG_ADDITIONAL_TEXT:
-		if (eel_strcmp (details->additional_text, GTK_VALUE_STRING (*arg)) == 0) {
+	case PROP_ADDITIONAL_TEXT:
+		if (eel_strcmp (details->additional_text,
+				g_value_get_string (value)) == 0) {
 			return;
 		}
 
 		g_free (details->additional_text);
-		details->additional_text = g_strdup (GTK_VALUE_STRING (*arg));
+		details->additional_text = g_strdup (g_value_get_string (value));
 		
 		nautilus_icon_canvas_item_invalidate_label_size (item);		
 		break;
 
-	case ARG_HIGHLIGHTED_FOR_SELECTION:
-		if (!details->is_highlighted_for_selection == !GTK_VALUE_BOOL (*arg)) {
+	case PROP_HIGHLIGHTED_FOR_SELECTION:
+		if (!details->is_highlighted_for_selection == !g_value_get_boolean (value)) {
 			return;
 		}
-		details->is_highlighted_for_selection = GTK_VALUE_BOOL (*arg);
+		details->is_highlighted_for_selection = g_value_get_boolean (value);
 		break;
          
-        case ARG_HIGHLIGHTED_AS_KEYBOARD_FOCUS:
-		if (!details->is_highlighted_as_keyboard_focus == !GTK_VALUE_BOOL (*arg)) {
+        case PROP_HIGHLIGHTED_AS_KEYBOARD_FOCUS:
+		if (!details->is_highlighted_as_keyboard_focus == !g_value_get_boolean (value)) {
 			return;
 		}
-		details->is_highlighted_as_keyboard_focus = GTK_VALUE_BOOL (*arg);
+		details->is_highlighted_as_keyboard_focus = g_value_get_boolean (value);
+
+		if (details->is_highlighted_as_keyboard_focus) {
+			AtkObject *atk_object = eel_accessibility_for_object (object);
+			atk_focus_tracker_notify (atk_object);
+		}
 		break;
 		
-        case ARG_HIGHLIGHTED_FOR_DROP:
-		if (!details->is_highlighted_for_drop == !GTK_VALUE_BOOL (*arg)) {
+        case PROP_HIGHLIGHTED_FOR_DROP:
+		if (!details->is_highlighted_for_drop == !g_value_get_boolean (value)) {
 			return;
 		}
-		details->is_highlighted_for_drop = GTK_VALUE_BOOL (*arg);
+		details->is_highlighted_for_drop = g_value_get_boolean (value);
 		break;
-
+		
 	default:
 		g_warning ("nautilus_icons_view_item_item_set_arg on unknown argument");
 		return;
@@ -326,38 +351,41 @@ nautilus_icon_canvas_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (object));
 }
 
-/* Get_arg handler for the icon item */
+/* Get property handler for the icon item */
 static void
-nautilus_icon_canvas_item_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
+nautilus_icon_canvas_item_get_property (GObject        *object,
+					guint           property_id,
+					GValue         *value,
+					GParamSpec     *pspec)
 {
 	NautilusIconCanvasItemDetails *details;
 	
 	details = NAUTILUS_ICON_CANVAS_ITEM (object)->details;
 	
-	switch (arg_id) {
+	switch (property_id) {
 		
-	case ARG_EDITABLE_TEXT:
-		GTK_VALUE_STRING (*arg) = g_strdup (details->editable_text);
+	case PROP_EDITABLE_TEXT:
+		g_value_set_string (value, details->editable_text);
 		break;
 
-	case ARG_ADDITIONAL_TEXT:
-		GTK_VALUE_STRING (*arg) = g_strdup (details->additional_text);
+	case PROP_ADDITIONAL_TEXT:
+		g_value_set_string (value, details->additional_text);
 		break;
 		
-        case ARG_HIGHLIGHTED_FOR_SELECTION:
-                GTK_VALUE_BOOL (*arg) = details->is_highlighted_for_selection;
+        case PROP_HIGHLIGHTED_FOR_SELECTION:
+		g_value_set_boolean (value, details->is_highlighted_for_selection);
                 break;
 		
-        case ARG_HIGHLIGHTED_AS_KEYBOARD_FOCUS:
-                GTK_VALUE_BOOL (*arg) = details->is_highlighted_as_keyboard_focus;
+        case PROP_HIGHLIGHTED_AS_KEYBOARD_FOCUS:
+		g_value_set_boolean (value, details->is_highlighted_as_keyboard_focus);
                 break;
 		
-        case ARG_HIGHLIGHTED_FOR_DROP:
-                GTK_VALUE_BOOL (*arg) = details->is_highlighted_for_drop;
+        case PROP_HIGHLIGHTED_FOR_DROP:
+		g_value_set_boolean (value, details->is_highlighted_for_drop);
                 break;
 
         default:
-		arg->type = G_TYPE_INVALID;
+		g_warning ("invalid property %d", property_id);
 		break;
 	}
 }
@@ -527,11 +555,9 @@ nautilus_icon_canvas_item_update_bounds (NautilusIconCanvasItem *item)
 		art_irect_union (&item->details->emblem_rect, &item->details->emblem_rect, &emblem_rect);
 	}
 
-	/* Send out the bounds_changed signal and queue a redraw. */
+	/* queue a redraw. */
 	eel_gnome_canvas_request_redraw_rectangle
 		(GNOME_CANVAS_ITEM (item)->canvas, before);
-	g_signal_emit (item,
-			 signals[BOUNDS_CHANGED], 0);
 	eel_gnome_canvas_item_request_redraw
 		(GNOME_CANVAS_ITEM (item));
 }
@@ -576,7 +602,7 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 	GnomeCanvasItem *canvas_item;
 	GdkPixbuf *selection_pixbuf;
 	PangoLayout *layout;
-	guint32 label_color;
+	GdkColor *label_color;
 	int layout_width, layout_height;
 	int icon_width;
 	gboolean have_editable, have_additional, needs_highlight;
@@ -631,6 +657,8 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 	height_so_far = 0;
 
 	max_text_width = floor (nautilus_icon_canvas_item_get_max_text_width (item));
+
+	container = NAUTILUS_ICON_CONTAINER (GNOME_CANVAS_ITEM (item)->canvas);	
 				
 	/* if the icon is highlighted, do some set-up */
 	if (needs_highlight && drawable != NULL && !details->is_renaming &&
@@ -641,17 +669,16 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 							   8,
 							   details->text_width,
 							   details->text_height);
-			container = NAUTILUS_ICON_CONTAINER (GNOME_CANVAS_ITEM (item)->canvas);	
 			eel_gdk_pixbuf_fill_rectangle_with_color (selection_pixbuf,
 								  eel_gdk_pixbuf_whole_pixbuf,
-								  container->details->highlight_color);
+								  container->details->highlight_color_rgba);
 			clear_rounded_corners (selection_pixbuf, container->details->highlight_frame, 5);
 			draw_pixbuf (selection_pixbuf, drawable, 
 				     icon_left + (icon_width - details->text_width) / 2,
 				     icon_bottom);
 			g_object_unref (selection_pixbuf);
 		} else {
-			gdk_rgb_gc_set_foreground (gc, highlight_background_color);
+			gdk_gc_set_rgb_fg_color (gc, &container->details->highlight_color);
 			
 			gdk_draw_rectangle
 				(drawable, GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas)->style->black_gc, TRUE,
@@ -666,13 +693,11 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 		layout = get_label_layout (&details->editable_text_layout, item, details->editable_text);
 
 		if (drawable != NULL) {
-			if (needs_highlight) {
-				label_color = highlight_text_color;
-			} else {
-				label_color = nautilus_icon_container_get_label_color (NAUTILUS_ICON_CONTAINER (canvas_item->canvas), TRUE);
-			}
+			label_color = nautilus_icon_container_get_label_color
+				(NAUTILUS_ICON_CONTAINER (canvas_item->canvas),
+				 TRUE, needs_highlight);
 
-			gdk_rgb_gc_set_foreground (gc, label_color);
+			gdk_gc_set_rgb_fg_color (gc, label_color);
 			
 			draw_label_layout (item, drawable,
 					   layout, needs_highlight,
@@ -692,13 +717,11 @@ draw_or_measure_label_text (NautilusIconCanvasItem *item,
 		layout = get_label_layout (&details->additional_text_layout, item, details->additional_text);
 
 		if (drawable != NULL) {
-			if (needs_highlight) {
-				label_color = highlight_text_info_color;
-			} else {
-				label_color = nautilus_icon_container_get_label_color (NAUTILUS_ICON_CONTAINER (canvas_item->canvas), FALSE);
-			}
+			label_color = nautilus_icon_container_get_label_color
+				(NAUTILUS_ICON_CONTAINER (canvas_item->canvas),
+				 FALSE, needs_highlight);
 
-			gdk_rgb_gc_set_foreground (gc, label_color);
+			gdk_gc_set_rgb_fg_color (gc, label_color);
 			
 			draw_label_layout (item, drawable,
 					   layout, needs_highlight,
@@ -818,7 +841,7 @@ draw_stretch_handles (NautilusIconCanvasItem *item, GdkDrawable *drawable,
 	g_free (knob_filename);
 	g_object_unref (knob_pixbuf);	
 
-	gdk_gc_unref (gc);
+	g_object_unref (gc);
 }
 
 static void
@@ -1136,14 +1159,31 @@ create_label_layout (NautilusIconCanvasItem *item,
 		     const char *text)
 {
 	PangoLayout *layout;
+	PangoContext *context;
+	PangoFontDescription *desc;
+	NautilusIconContainer *container;
 
-	layout = pango_layout_new (eel_gnome_canvas_get_pango_context (GNOME_CANVAS_ITEM (item)->canvas));
+	container = NAUTILUS_ICON_CONTAINER (GNOME_CANVAS_ITEM (item)->canvas);
+	context = eel_gnome_canvas_get_pango_context (GNOME_CANVAS_ITEM (item)->canvas);
+	layout = pango_layout_new (context);
 
 	pango_layout_set_text (layout, text, -1);
 	pango_layout_set_width (layout, floor (nautilus_icon_canvas_item_get_max_text_width (item)) * PANGO_SCALE);
 	pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
 	pango_layout_set_spacing (layout, LABEL_LINE_SPACING);
 
+	/* Create a font description */
+	if (container->details->font) {
+		desc = pango_font_description_from_string (container->details->font);
+	} else {
+		desc = pango_font_description_copy (pango_context_get_font_description (context));
+		pango_font_description_set_size (desc,
+						 pango_font_description_get_size (desc) +
+						 container->details->font_size_table [container->details->zoom_level]);
+	}
+	pango_layout_set_font_description (layout, desc);
+	pango_font_description_free (desc);
+	
 	/* if it's prelit, and we're in click-to-activate mode, underline the text */
 	if (item->details->is_prelit && in_single_click_mode ()) {
 		eel_pango_layout_set_underline (layout, PANGO_UNDERLINE_SINGLE);
@@ -1664,49 +1704,317 @@ nautilus_icon_canvas_item_get_max_text_width (NautilusIconCanvasItem *item)
 
 }
 
+static G_CONST_RETURN gchar *
+nautilus_icon_canvas_item_accessible_get_name (AtkObject *accessible)
+{
+	NautilusIconCanvasItem *item;
+
+	item = eel_accessibility_get_gobject (accessible);
+	if (!item) {
+		return NULL;
+	}
+
+	return item->details->editable_text;
+}
+
+static G_CONST_RETURN gchar*
+nautilus_icon_canvas_item_accessible_get_description (AtkObject *accessible)
+{
+	NautilusIconCanvasItem *item;
+
+	item = eel_accessibility_get_gobject (accessible);
+	if (!item) {
+		return NULL;
+	}
+
+	return item->details->additional_text;
+}
+
+static AtkObject *
+nautilus_icon_canvas_item_accessible_get_parent (AtkObject *accessible)
+{
+	NautilusIconCanvasItem *item;
+	
+	item = eel_accessibility_get_gobject (accessible);
+	if (!item) {
+		return NULL;
+	}
+
+	return gtk_widget_get_accessible (GTK_WIDGET (GNOME_CANVAS_ITEM (item)->canvas));
+}
+
+static int
+nautilus_icon_canvas_item_accessible_get_index_in_parent (AtkObject *accessible)
+{
+	NautilusIconCanvasItem *item;
+	NautilusIconContainer *container;
+	GList *l;
+	NautilusIcon *icon;
+	int i;
+
+	item = eel_accessibility_get_gobject (accessible);
+	if (!item) {
+		return -1;
+	}
+	
+	container = NAUTILUS_ICON_CONTAINER (GNOME_CANVAS_ITEM (item)->canvas);
+	
+	l = container->details->icons;
+	i = 0;
+	while (l) {
+		icon = l->data;
+		
+		if (icon->item == item) {
+			return i;
+		}
+		
+		i++;
+		l = l->next;
+	}
+
+	return -1;
+}
+
+
+static void
+nautilus_icon_canvas_item_accessible_class_init (AtkObjectClass *klass)
+{
+	klass->get_name = nautilus_icon_canvas_item_accessible_get_name;
+	klass->get_description = nautilus_icon_canvas_item_accessible_get_description;
+	klass->get_parent = nautilus_icon_canvas_item_accessible_get_parent;
+	klass->get_index_in_parent = nautilus_icon_canvas_item_accessible_get_index_in_parent;
+}
+
+
+static G_CONST_RETURN gchar * 
+nautilus_icon_canvas_item_accessible_get_image_description
+	(AtkImage *image)
+{
+	return _("file icon");
+}
+
+
+static void
+nautilus_icon_canvas_item_accessible_get_image_size
+	(AtkImage *image, 
+	 gint     *width,
+	 gint     *height)
+{
+	NautilusIconCanvasItem *item;
+
+	item = eel_accessibility_get_gobject (ATK_OBJECT (image));
+
+	if (!item || !item->details->pixbuf) {
+		*width = *height = 0;
+	} else {
+		*width = gdk_pixbuf_get_width (item->details->pixbuf);
+		*height = gdk_pixbuf_get_height (item->details->pixbuf);
+	}
+}
+
+static void
+nautilus_icon_canvas_item_accessible_get_image_position
+	(AtkImage		 *image,
+	 gint                    *x,
+	 gint	                 *y,
+	 AtkCoordType	         coord_type)
+{
+	AtkComponentIface *component_if;
+
+	component_if = g_type_interface_peek (image, ATK_TYPE_COMPONENT);
+
+	component_if->get_position
+		(ATK_COMPONENT (image), x, y, coord_type);
+}
+
+static gboolean
+nautilus_icon_canvas_item_accessible_set_image_description
+	(AtkImage    *image,
+	 const gchar *description)
+{
+	g_warning (G_STRLOC "this api seems broken");
+	return FALSE;
+}
+
+static void
+nautilus_icon_canvas_item_accessible_image_interface_init (AtkImageIface *iface)
+{
+	iface->get_image_description = nautilus_icon_canvas_item_accessible_get_image_description;
+	iface->set_image_description = nautilus_icon_canvas_item_accessible_set_image_description;
+	iface->get_image_size        = nautilus_icon_canvas_item_accessible_get_image_size;
+	iface->get_image_position    = nautilus_icon_canvas_item_accessible_get_image_position;
+}
+
+static GType
+nautilus_icon_canvas_item_accessible_get_type (void)
+{
+	static GType type = 0;
+
+	if (!type) {
+		static const GInterfaceInfo atk_image_info = {
+			(GInterfaceInitFunc)
+			nautilus_icon_canvas_item_accessible_image_interface_init,
+			(GInterfaceFinalizeFunc) NULL,
+			NULL
+		};
+		type = eel_accessibility_create_derived_type (
+			"NautilusIconCanvasItemAccessibility",
+			GNOME_TYPE_CANVAS_ITEM,
+			nautilus_icon_canvas_item_accessible_class_init);
+
+		if (type != G_TYPE_INVALID) {
+			g_type_add_interface_static (
+				type, ATK_TYPE_IMAGE, &atk_image_info);
+
+			eel_accessibility_add_simple_text (type);
+		}
+	}
+
+	return type;
+}
+
+static AtkObject *
+nautilus_icon_canvas_item_accessible_create (GObject *for_object)
+{
+	GType type;
+	AtkObject *accessible;
+	NautilusIconCanvasItem *item;
+
+	item = NAUTILUS_ICON_CANVAS_ITEM (for_object);
+	g_return_val_if_fail (item != NULL, NULL);
+
+	type = nautilus_icon_canvas_item_accessible_get_type ();
+
+	if (type == G_TYPE_INVALID) {
+		return atk_no_op_object_new (for_object);
+	}
+
+	item->details->text_util = gail_text_util_new ();
+	gail_text_util_text_setup (item->details->text_util,
+				   item->details->editable_text);
+
+	accessible = g_object_new (type, NULL);
+
+	return eel_accessibility_set_atk_object_return
+		(for_object, accessible);
+}
+
+EEL_ACCESSIBLE_FACTORY (nautilus_icon_canvas_item_accessible_get_type (),
+			"NautilusIconCanvasItemAccessibilityFactory",
+			nautilus_icon_canvas_item_accessible,
+			nautilus_icon_canvas_item_accessible_create);
+
+
+static GailTextUtil *
+nautilus_icon_canvas_item_get_text (GObject *text)
+{
+	return NAUTILUS_ICON_CANVAS_ITEM (text)->details->text_util;
+}
+
+static void
+nautilus_icon_canvas_item_text_interface_init (EelAccessibleTextIface *iface)
+{
+	iface->get_text = nautilus_icon_canvas_item_get_text;
+}
+
 /* Class initialization function for the icon canvas item. */
 static void
 nautilus_icon_canvas_item_class_init (NautilusIconCanvasItemClass *class)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	GnomeCanvasItemClass *item_class;
 
-	object_class = GTK_OBJECT_CLASS (class);
+	parent_class = g_type_class_peek_parent (class);
+
+	object_class = G_OBJECT_CLASS (class);
 	item_class = GNOME_CANVAS_ITEM_CLASS (class);
 
-	gtk_object_add_arg_type	("NautilusIconCanvasItem::editable_text",
-				 G_TYPE_STRING, GTK_ARG_READWRITE, ARG_EDITABLE_TEXT);
-	gtk_object_add_arg_type	("NautilusIconCanvasItem::additional_text",
-				 G_TYPE_STRING, GTK_ARG_READWRITE, ARG_ADDITIONAL_TEXT);
-	gtk_object_add_arg_type	("NautilusIconCanvasItem::highlighted_for_selection",
-				 G_TYPE_BOOLEAN, GTK_ARG_READWRITE, ARG_HIGHLIGHTED_FOR_SELECTION);
-	gtk_object_add_arg_type	("NautilusIconCanvasItem::highlighted_as_keyboard_focus",
-				 G_TYPE_BOOLEAN, GTK_ARG_READWRITE, ARG_HIGHLIGHTED_AS_KEYBOARD_FOCUS);
-	gtk_object_add_arg_type	("NautilusIconCanvasItem::highlighted_for_drop",
-				 G_TYPE_BOOLEAN, GTK_ARG_READWRITE, ARG_HIGHLIGHTED_FOR_DROP);
+	object_class->finalize = nautilus_icon_canvas_item_finalize;
+	object_class->set_property = nautilus_icon_canvas_item_set_property;
+	object_class->get_property = nautilus_icon_canvas_item_get_property;
 
-	G_OBJECT_CLASS (class)->finalize = nautilus_icon_canvas_item_finalize;
+        g_object_class_install_property (
+		object_class,
+		PROP_EDITABLE_TEXT,
+		g_param_spec_string ("editable_text",
+				     _("editable text"),
+				     _("the editable label"),
+				     "", G_PARAM_READWRITE));
 
-	object_class->set_arg = nautilus_icon_canvas_item_set_arg;
-	object_class->get_arg = nautilus_icon_canvas_item_get_arg;
+        g_object_class_install_property (
+		object_class,
+		PROP_ADDITIONAL_TEXT,
+		g_param_spec_string ("additional_text",
+				     _("additional text"),
+				     _("some more text"),
+				     "", G_PARAM_READWRITE));
 
-	signals[BOUNDS_CHANGED]
-		= g_signal_new ("bounds_changed",
-		                G_TYPE_FROM_CLASS (class),
-		                G_SIGNAL_RUN_LAST,
-		                G_STRUCT_OFFSET (NautilusIconCanvasItemClass,
-						     bounds_changed),
-		                NULL, NULL,
-		                gtk_marshal_VOID__VOID,
-		                G_TYPE_NONE, 0);
+        g_object_class_install_property (
+		object_class,
+		PROP_HIGHLIGHTED_FOR_SELECTION,
+		g_param_spec_boolean ("highlighted_for_selection",
+				      _("highlighted for selection"),
+				      _("whether we are highlighted for a selection"),
+				      FALSE, G_PARAM_READWRITE)); 
+
+        g_object_class_install_property (
+		object_class,
+		PROP_HIGHLIGHTED_AS_KEYBOARD_FOCUS,
+		g_param_spec_boolean ("highlighted_as_keyboard_focus",
+				      _("highlighted as keyboard focus"),
+				      _("whether we are highlighted to render keyboard focus"),
+				      FALSE, G_PARAM_READWRITE)); 
+
+
+        g_object_class_install_property (
+		object_class,
+		PROP_HIGHLIGHTED_FOR_DROP,
+		g_param_spec_boolean ("highlighted_for_drop",
+				      _("highlighted for drop"),
+				      _("whether we are highlighted for a D&D drop"),
+				      FALSE, G_PARAM_READWRITE)); 
 
 	item_class->update = nautilus_icon_canvas_item_update;
 	item_class->draw = nautilus_icon_canvas_item_draw;
 	item_class->render = nautilus_icon_canvas_item_render;
 	item_class->point = nautilus_icon_canvas_item_point;
 	item_class->bounds = nautilus_icon_canvas_item_bounds;
-	item_class->event = nautilus_icon_canvas_item_event;
+	item_class->event = nautilus_icon_canvas_item_event;	
 
-	eel_preferences_add_auto_integer (NAUTILUS_PREFERENCES_CLICK_POLICY,
-					  &click_policy_auto_value);
+	EEL_OBJECT_SET_FACTORY (NAUTILUS_TYPE_ICON_CANVAS_ITEM,
+				nautilus_icon_canvas_item_accessible);
+}
+
+GType
+nautilus_icon_canvas_item_get_type (void)
+{
+	static GType type = 0;
+
+	if (!type) {
+		static const GTypeInfo info = {
+			sizeof (NautilusIconCanvasItemClass),
+			NULL,		/* base_init */
+			NULL,		/* base_finalize */
+			(GClassInitFunc) nautilus_icon_canvas_item_class_init,
+			NULL,		/* class_finalize */
+			NULL,               /* class_data */
+			sizeof (NautilusIconCanvasItem),
+			0,                  /* n_preallocs */
+			(GInstanceInitFunc) nautilus_icon_canvas_item_init,
+		};
+		static const GInterfaceInfo eel_text_info = {
+			(GInterfaceInitFunc)
+			nautilus_icon_canvas_item_text_interface_init,
+			(GInterfaceFinalizeFunc) NULL,
+			NULL
+		};
+
+		type = g_type_register_static
+			(GNOME_TYPE_CANVAS_ITEM, "NautilusIconCanvasItem", &info, 0);
+
+		g_type_add_interface_static
+			(type, EEL_TYPE_ACCESSIBLE_TEXT, &eel_text_info);
+	}
+
+	return type;
 }
