@@ -48,6 +48,7 @@
 #include <eel/eel-pango-extensions.h>
 #include <eel/eel-string.h>
 #include <eel/eel-vfs-extensions.h>
+#include <gtk/gtksettings.h>
 #include <gtk/gtksignal.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
@@ -221,6 +222,7 @@ typedef struct {
 
 	/* Used to know when to make a new thumbnail. */
 	time_t cache_time;
+	GnomeVFSFileSize cache_size;
 
 	/* Type of icon. */
 	IconRequest request;
@@ -1513,7 +1515,9 @@ path_represents_svg_image (const char *path)
 
 /* Returns GNOME_VFS_ERROR_NOT_SUPPORTED for icons that are not files. */
 static GnomeVFSResult
-get_cache_time (const char *file_uri, time_t *cache_time)
+get_cache_time_and_size (const char *file_uri,
+			 time_t *cache_time,
+			 GnomeVFSFileSize *cache_size)
 {
 	GnomeVFSURI *vfs_uri;
 	GnomeVFSFileInfo *file_info;
@@ -1540,6 +1544,7 @@ get_cache_time (const char *file_uri, time_t *cache_time)
 	result = gnome_vfs_get_file_info (file_uri, file_info, GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
 	if (result == GNOME_VFS_OK) {
 		*cache_time = file_info->mtime;
+		*cache_size = file_info->size;
 	}
 	gnome_vfs_file_info_unref (file_info);
 
@@ -1665,7 +1670,7 @@ load_specific_icon (NautilusScalableIcon *scalable_icon,
 
 	/* Since we got something, we can create a cache icon. */
 	icon = cache_icon_new (pixbuf, type, FALSE, &details);
-	get_cache_time (scalable_icon->uri, &icon->cache_time);
+	get_cache_time_and_size (scalable_icon->uri, &icon->cache_time, &icon->cache_size);
 	g_object_unref (pixbuf);
 
 	return icon;
@@ -1796,7 +1801,7 @@ scale_icon (CacheIcon *icon,
 	rect_height = (icon->details.text_rect.y1 - icon->details.text_rect.y0) * scale_y;
 	
 	scaled_details.text_rect.x0 = icon->details.text_rect.x0 * scale_x;
-	scaled_details.text_rect.y0 = icon->details.text_rect.x0 * scale_y;
+	scaled_details.text_rect.y0 = icon->details.text_rect.y0 * scale_y;
 	scaled_details.text_rect.x1 = scaled_details.text_rect.x0 + rect_width;
 	scaled_details.text_rect.y1 = scaled_details.text_rect.y0 + rect_height;
 
@@ -1813,6 +1818,7 @@ scale_icon (CacheIcon *icon,
 				      &scaled_details);
 	scaled_icon->is_fallback = icon->is_fallback;
 	scaled_icon->cache_time = icon->cache_time;
+	scaled_icon->cache_size = icon->cache_size;
 	g_object_unref (scaled_pixbuf);
 	return scaled_icon;
 }
@@ -1939,13 +1945,14 @@ mark_recently_used (CircularList *node)
  * from the cache if the icon has changed.
  */
 static gboolean
-remove_icons_if_file_changed (const char *file_uri, time_t cached_time)
+remove_icons_if_file_changed (const char *file_uri, time_t cached_time, GnomeVFSFileSize cached_size)
 {
 	GnomeVFSResult result;
 	time_t new_time;
-
+	GnomeVFSFileSize new_size;
+	
 	/* Get the time from the file. */
-	result = get_cache_time (file_uri, &new_time);
+	result = get_cache_time_and_size (file_uri, &new_time, &new_size);
 
 	/* Do nothing for cases where a time doesn't apply. */
 	if (result == GNOME_VFS_ERROR_NOT_SUPPORTED) {
@@ -1953,7 +1960,7 @@ remove_icons_if_file_changed (const char *file_uri, time_t cached_time)
 	}
 
 	/* Do nothing if the file is still the same as before. */
-	if (result == GNOME_VFS_OK && new_time == cached_time) {
+	if (result == GNOME_VFS_OK && new_time == cached_time && new_size == cached_size) {
 		return FALSE;
 	}
 
@@ -2008,7 +2015,8 @@ get_icon_from_cache (NautilusScalableIcon *scalable_icon,
 
 		/* Check if the cached image is good before using it. */
 		if (remove_icons_if_file_changed (scalable_icon->uri,
-						  icon->cache_time)) {
+						  icon->cache_time,
+						  icon->cache_size)) {
 			icon = NULL;
 		}
 	}
@@ -2248,7 +2256,9 @@ static gboolean embedded_text_preferences_callbacks_added = FALSE;
 static PangoFontDescription *embedded_text_font = NULL;
 
 static void
-embedded_text_font_changed_callback (gpointer callback_data)
+embedded_text_font_changed_callback (GtkSettings *settings,
+				     GParamSpec  *pspec,
+				     gpointer     callback_data)
 {
 	gboolean clear_cache;
 	char *font_name;
@@ -2259,7 +2269,15 @@ embedded_text_font_changed_callback (gpointer callback_data)
 		pango_font_description_free (embedded_text_font);
 	}
 
-	font_name = eel_preferences_get (NAUTILUS_PREFERENCES_DEFAULT_FONT);
+	font_name = NULL;
+	g_object_get (settings,
+		      "gtk-font-name", &font_name,
+		      NULL);
+
+	if (font_name == NULL) {
+		font_name = g_strdup ("Sans");
+	}
+	
 	embedded_text_font = pango_font_description_from_string (font_name);
 	g_free (font_name);
 
@@ -2279,6 +2297,7 @@ embed_text (GdkPixbuf *pixbuf_without_text,
 	GdkPixbuf *pixbuf_with_text;
 	PangoLayout *layout;
 	static PangoContext *context;
+	GtkSettings *settings;
 	
 	g_return_val_if_fail (pixbuf_without_text != NULL, NULL);
 	
@@ -2293,10 +2312,10 @@ embed_text (GdkPixbuf *pixbuf_without_text,
 	if (!embedded_text_preferences_callbacks_added) {
 		embedded_text_preferences_callbacks_added = TRUE;
 
-		eel_preferences_add_callback (NAUTILUS_PREFERENCES_DEFAULT_FONT,
-					      embedded_text_font_changed_callback,
-					      GINT_TO_POINTER (TRUE));
-		embedded_text_font_changed_callback (GINT_TO_POINTER (FALSE));
+		settings = gtk_settings_get_default ();	
+		g_signal_connect (settings, "notify::gtk-font-name",
+				  G_CALLBACK (embedded_text_font_changed_callback), GINT_TO_POINTER (TRUE));
+		embedded_text_font_changed_callback (settings, NULL, GINT_TO_POINTER (FALSE));
 	}
 	
 	if (context == NULL) {
@@ -2354,6 +2373,7 @@ load_icon_with_embedded_text (NautilusScalableIcon *scalable_icon,
 					 icon_without_text->scaled,
 					 &details);
 	icon_with_text->cache_time = icon_without_text->cache_time;
+	icon_with_text->cache_size = icon_without_text->cache_size;
 	cache_icon_unref (icon_without_text);
 	g_object_unref (pixbuf_with_text);
 
